@@ -88,29 +88,34 @@ class AwsKinesisStream(AwsResource):
     kinesis_encryption_type: Optional[str] = field(default=None)
     kinesis_key_id: Optional[str] = field(default=None)
 
+    @classmethod
+    def called_collect_apis(cls) -> List[AwsApiSpec]:
+        return [cls.api_spec, AwsApiSpec("kinesis", "describe-stream"), AwsApiSpec("kinesis", "list-tags-for-stream")]
+
     @classmethod
     def collect(cls: Type[AwsResource], json: List[Json], builder: GraphBuilder) -> None:
         def add_tags(stream: AwsKinesisStream) -> None:
-            tags = builder.client.list(stream.api_spec.service, "list_tags_for_stream", None, StreamName=stream.name)
+            tags = builder.client.list(stream.api_spec.service, "list-tags-for-stream", "Tags", StreamName=stream.name)
             if tags:
-                stream.tags = bend(S("Tags", default=[]) >> ToDict(), tags[0])
+                stream.tags = bend(ToDict(), tags)
 
         for stream_name in json:
-            stream_description = builder.client.get(
-                service="kinesis",
+            # this call is paginated and will return a list
+            stream_descriptions = builder.client.list(
+                aws_service="kinesis",
                 action="describe-stream",
                 result_name="StreamDescription",
                 StreamName=stream_name,
             )
-            if stream_description is not None:
-                stream = AwsKinesisStream.from_api(stream_description)
+            if len(stream_descriptions) == 1:
+                stream = AwsKinesisStream.from_api(stream_descriptions[0])
                 builder.add_node(stream)
                 builder.submit_work(add_tags, stream)
 
     def update_resource_tag(self, client: AwsClient, key: str, value: str) -> bool:
         client.call(
-            service=self.api_spec.service,
-            action="add_tags_to_stream",
+            aws_service=self.api_spec.service,
+            action="add-tags-to-stream",
             result_name=None,
             StreamName=self.name,
             Tags={key: value},
@@ -119,8 +124,8 @@ class AwsKinesisStream(AwsResource):
 
     def delete_resource_tag(self, client: AwsClient, key: str) -> bool:
         client.call(
-            service=self.api_spec.service,
-            action="remove_tags_from_stream",
+            aws_service=self.api_spec.service,
+            action="remove-tags-from-stream",
             result_name=None,
             StreamName=self.name,
             TagKeys=[key],
@@ -129,12 +134,20 @@ class AwsKinesisStream(AwsResource):
 
     def delete_resource(self, client: AwsClient) -> bool:
         client.call(
-            service=self.api_spec.service,
-            action="delete_stream",
+            aws_service=self.api_spec.service,
+            action="delete-stream",
             result_name=None,
             StreamName=self.name,
         )
         return True
 
+    @classmethod
+    def called_mutator_apis(cls) -> List[AwsApiSpec]:
+        return [
+            AwsApiSpec("kinesis", "add-tags-to-stream"),
+            AwsApiSpec("kinesis", "remove-tags-from-stream"),
+            AwsApiSpec("kinesis", "delete-stream"),
+        ]
+
 
 resources: List[Type[AwsResource]] = [AwsKinesisStream]
