@@ -4,22 +4,23 @@ from attr import define, field
 from resoto_plugin_aws.resource.base import AwsApiSpec, AwsResource, GraphBuilder
 from resoto_plugin_aws.resource.cloudwatch import AwsCloudwatchQuery, AwsCloudwatchMetricData
 from resoto_plugin_aws.resource.ec2 import AwsEc2SecurityGroup, AwsEc2Subnet, AwsEc2Vpc
+from resoto_plugin_aws.resource.kms import AwsKmsKey
 from resoto_plugin_aws.utils import ToDict
-from resotolib.baseresources import BaseAccount, BaseDatabase, ModelReference  # noqa: F401
-from resotolib.json_bender import F, K, S, Bend, Bender, ForallBend
+from resotolib.baseresources import BaseDatabase, ModelReference
+from resotolib.json_bender import F, K, S, Bend, Bender, ForallBend, bend
 from resotolib.types import Json
 from resotolib.utils import utc
 from resoto_plugin_aws.aws_client import AwsClient
 
 
-# todo: annotate with no serialization annotation
+# noinspection PyUnresolvedReferences
 class RdsTaggable:
     def update_resource_tag(self, client: AwsClient, key: str, value: str) -> bool:
         if isinstance(self, AwsResource):
             if spec := self.api_spec:
                 client.call(
-                    service=spec.service,
-                    action="add_tags_to_resource",
+                    aws_service=spec.service,
+                    action="add-tags-to-resource",
                     result_name=None,
                     ResourceName=self.arn,
                     Tags=[{"Key": key, "Value": value}],
@@ -32,8 +33,8 @@ class RdsTaggable:
         if isinstance(self, AwsResource):
             if spec := self.api_spec:
                 client.call(
-                    service=spec.service,
-                    action="remove_tags_from_resource",
+                    aws_service=spec.service,
+                    action="remove-tags-from-resource",
                     result_name=None,
                     ResourceName=self.arn,
                     TagKeys=[key],
@@ -42,6 +43,10 @@ class RdsTaggable:
             return False
         return False
 
+    @classmethod
+    def called_mutator_apis(cls) -> List[AwsApiSpec]:
+        return [AwsApiSpec("rds", "add-tags-to-resource"), AwsApiSpec("rds", "remove-tags-from-resource")]
+
 
 @define(eq=False, slots=False)
 class AwsRdsEndpoint:
@@ -248,8 +253,9 @@ class AwsRdsInstance(RdsTaggable, AwsResource, BaseDatabase):
     reference_kinds: ClassVar[ModelReference] = {
         "predecessors": {
             "default": ["aws_vpc", "aws_ec2_security_group", "aws_ec2_subnet"],
-            "delete": ["aws_vpc", "aws_ec2_security_group", "aws_ec2_subnet"],
+            "delete": ["aws_vpc", "aws_ec2_security_group", "aws_ec2_subnet", "aws_kms_key"],
         },
+        "successors": {"default": ["aws_kms_key"]},
     }
     mapping: ClassVar[Dict[str, Bender]] = {
         "id": S("DBInstanceIdentifier"),
@@ -405,8 +411,17 @@ class AwsRdsInstance(RdsTaggable, AwsResource, BaseDatabase):
     rds_backup_target: Optional[str] = field(default=None)
     rds_network_type: Optional[str] = field(default=None)
 
+    @classmethod
+    def called_collect_apis(cls) -> List[AwsApiSpec]:
+        return [cls.api_spec, AwsApiSpec(cls.api_spec.service, "list-tags-for-resource")]
+
     @classmethod
     def collect(cls: Type[AwsResource], json: List[Json], builder: GraphBuilder) -> None:
+        def add_tags(rds: AwsRdsInstance) -> None:
+            tags = builder.client.list("rds", "list-tags-for-resource", "TagList", ResourceName=rds.arn)
+            if tags:
+                rds.tags = bend(ToDict(), tags)
+
         instances: List[AwsRdsInstance] = []
 
         def update_atime_mtime() -> None:
@@ -441,10 +456,10 @@ class AwsRdsInstance(RdsTaggable, AwsResource, BaseDatabase):
             instance = AwsRdsInstance.from_api(js)
             instances.append(instance)
             builder.add_node(instance, js)
+            builder.submit_work(add_tags, instance)
         update_atime_mtime()
 
     def connect_in_graph(self, builder: GraphBuilder, source: Json) -> None:
-        super().connect_in_graph(builder, source)
         for group in self.rds_vpc_security_groups:
             builder.dependant_node(
                 self,
@@ -463,16 +478,28 @@ class AwsRdsInstance(RdsTaggable, AwsResource, BaseDatabase):
                 builder.dependant_node(
                     self, reverse=True, delete_same_as_default=True, clazz=AwsEc2Subnet, id=subnet_id
                 )
+        potential_keys = [
+            self.rds_kms_key_id,
+            self.rds_performance_insights_kms_key_id,
+            self.rds_activity_stream_kms_key_id,
+        ]
+        keys = [key for key in potential_keys if key]
+        for key_reference in keys:
+            builder.dependant_node(from_node=self, clazz=AwsKmsKey, id=AwsKmsKey.normalise_id(key_reference))
 
     def delete_resource(self, client: AwsClient) -> bool:
         client.call(
-            service=self.api_spec.service,
-            action="delete_db_instance",
+            aws_service=self.api_spec.service,
+            action="delete-db-instance",
             result_name=None,
             DBInstanceIdentifier=self.id,
             SkipFinalSnapshot=True,
         )
         return True
 
+    @classmethod
+    def called_mutator_apis(cls) -> List[AwsApiSpec]:
+        return super().called_mutator_apis() + [AwsApiSpec("rds", "delete-db-instance")]
+
 
 resources: List[Type[AwsResource]] = [AwsRdsInstance]
